new;
library pgraph;
cls;

/* This program computes the coverage probability
** of the frequentist confidence interval for different
** values of n and lambda
*/

format /mat /on /mb1 /ros 8,3;
tau = 0.10;

n_grid  = 10 | 50 | 100 | 500;
la_grid = 0.01 | 0.25 | 1 | 10;

hpd_n = 1001;

critval_min = abs(cdfni(tau))-0.1;
critval_max = abs(cdfni(tau/2))+0.1;
critval_n = 101;
critval_grid = seqa(critval_min,(critval_max-critval_min)/(critval_n-1),critval_n);

n_n = rows(n_grid);
la_n = rows(la_grid);

bayescov_mat    = zeros(n_n,la_n);
freqlength_mat  = zeros(n_n,la_n);
bayeslength_mat = zeros(n_n,la_n);

n_ind = 1;
do until n_ind > n_n;

   la_ind = 1;
   do until la_ind > la_n;
   
      lam = la_grid[la_ind];
      n = n_grid[n_ind];
   
      /* compute critical value for freq interval */
      critval_obj = ( cdfn(lam*sqrt(n)+critval_grid)-cdfn(-critval_grid) 
                    - (1-tau) )^2;
      critval = critval_grid[minindc(critval_obj)];
      
      /* evaluate posterior cdf at boundaries of freq interval
      ** without loss of generality we assume phi-hat = 0
      */
      lb_freq = -critval/sqrt(n);
      ub_freq = lam+critval/sqrt(n);
      
      /* construct the HP interval
      */
      hpd_min  = lb_freq;
      hpd_max  = lam/2;
      hpd_grid = seqa(hpd_min,(hpd_max-hpd_min)/(hpd_n-1),hpd_n);
      
      hpd_mat = zeros(hpd_n,1);
      hpd_ind = 1;
      do until hpd_ind > hpd_n;
         hpd_mat[hpd_ind] = postcdf(hpd_grid[hpd_ind],lam,n);
         hpd_ind = hpd_ind+1;
      endo;
      lb_hpd = hpd_grid[minindc( (hpd_mat-tau/2)^2 )];
      ub_hpd = lam/2 + (lam/2-lb_hpd);


      /* store results */
      bayescov_mat[n_ind,la_ind]   = postcdf(ub_freq,lam,n)-postcdf(lb_freq,lam,n);
      freqlength_mat[n_ind,la_ind] = ub_freq-lb_freq;
      bayeslength_mat[n_ind,la_ind] = ub_hpd-lb_hpd;
      
      la_ind = la_ind+1;
      
   endo;
   n_ind = n_ind+1;
   
endo;

bayescov_mat;

freqlength_mat;

bayeslength_mat;

end;

proc (1) = postcdf(x,la,n);
local f;
f = (1/la)*( x*cdfn(sqrt(n)*x) - (x-la)*cdfn(sqrt(n)*(x-la)) )
  + (1/(la*sqrt(n)))*( pdfn(sqrt(n)*x) - pdfn(sqrt(n)*(x-la)) );
retp(f);
endp; 
